@@ -40,7 +40,9 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpReserved)
 	return TRUE;
 }
 #elif HAVE_CARBON
-#include <Carbon/Carbon.h>
+  #ifdef ENABLE_BINRELOC
+    #include <Carbon/Carbon.h>
+  #endif
 #endif
 
 
@@ -71,58 +73,105 @@ gda_gbr_get_file_path (GdaPrefixDir where, ...)
 	gchar **parts;
 	gint size, i;
 	const gchar *prefix_dir_name = NULL;
+	gint prefix_len = strlen (LIBGDAPREFIX);
+
+	/*
+	g_print ("LIBGDAPREFIX = %s\n", LIBGDAPREFIX);
+	g_print ("LIBGDABIN = %s\n", LIBGDABIN);
+	g_print ("LIBGDASBIN = %s\n", LIBGDASBIN);
+	g_print ("LIBGDADATA = %s\n",LIBGDADATA );
+	g_print ("LIBGDALIB = %s\n", LIBGDALIB);
+	g_print ("LIBGDALIBEXEC = %s\n",LIBGDALIBEXEC );
+	g_print ("LIBGDASYSCONF = %s\n", LIBGDASYSCONF);
+	*/
 
 #ifdef G_OS_WIN32
 	wchar_t path[MAX_PATH];
 	gchar* p;
 #endif
+
 	switch (where) {
 	default:
 	case GDA_NO_DIR:
 		break;
 	case GDA_BIN_DIR:
-		if (! g_str_has_prefix (LIBGDABIN, LIBGDAPREFIX)) 
-			prefix = g_strdup (LIBGDABIN);
+		tmp = LIBGDABIN;
+#ifndef G_OS_WIN32
+		if (! g_str_has_prefix (tmp, LIBGDAPREFIX) || (tmp [prefix_len] != G_DIR_SEPARATOR)) 
+			prefix = g_strdup (tmp);
 		else
-			prefix_dir_name = "bin";
+			prefix_dir_name = tmp + prefix_len + 1;
+#else
+		prefix_dir_name = "bin";
+#endif
 		break;
 	case GDA_SBIN_DIR:
-		if (! g_str_has_prefix (LIBGDASBIN, LIBGDAPREFIX)) 
-			prefix = g_strdup (LIBGDASBIN);
+		tmp = LIBGDASBIN;
+#ifndef G_OS_WIN32
+		if (! g_str_has_prefix (tmp, LIBGDAPREFIX) || (tmp [prefix_len] != G_DIR_SEPARATOR)) 
+			prefix = g_strdup (tmp);
 		else
-			prefix_dir_name = "sbin";
+			prefix_dir_name = tmp + prefix_len + 1;
+#else
+		prefix_dir_name = "sbin";
+#endif
 		break;
 	case GDA_DATA_DIR:
-		if (! g_str_has_prefix (LIBGDADATA, LIBGDAPREFIX)) 
-			prefix = g_strdup (LIBGDADATA);
+		tmp = LIBGDADATA;
+#ifndef G_OS_WIN32
+		if (! g_str_has_prefix (tmp, LIBGDAPREFIX) || (tmp [prefix_len] != G_DIR_SEPARATOR)) 
+			prefix = g_strdup (tmp);
 		else
-			prefix_dir_name = "share";
+			prefix_dir_name = tmp + prefix_len + 1;
+#else
+		prefix_dir_name = "share";
+#endif		
 		break;
 	case GDA_LOCALE_DIR:
-		if (! g_str_has_prefix (LIBGDADATA, LIBGDAPREFIX)) {
-			prefix = g_strdup (LIBGDADATA);
+		tmp = LIBGDADATA;
+#ifndef G_OS_WIN32
+		if (! g_str_has_prefix (tmp, LIBGDAPREFIX) || (tmp [prefix_len] != G_DIR_SEPARATOR)) {
+			prefix = g_strdup (tmp);
 			prefix_dir_name = "locale";
 		}
 		else
 			prefix_dir_name = "share" G_DIR_SEPARATOR_S "locale";
+#else
+		prefix_dir_name = "share" G_DIR_SEPARATOR_S "locale";
+#endif
 		break;
 	case GDA_LIB_DIR:
-		if (! g_str_has_prefix (LIBGDALIB, LIBGDAPREFIX)) 
-			prefix = g_strdup (LIBGDALIB);
-		else
-			prefix_dir_name = "lib";		
+		tmp = LIBGDALIB;
+#ifndef G_OS_WIN32
+		if (! g_str_has_prefix (tmp, LIBGDAPREFIX) || (tmp [prefix_len] != G_DIR_SEPARATOR)) 
+			prefix = g_strdup (tmp);
+		else 
+			prefix_dir_name = tmp + prefix_len + 1;
+#else
+		prefix_dir_name = "lib";
+#endif
 		break;
 	case GDA_LIBEXEC_DIR:
-		if (! g_str_has_prefix (LIBGDALIBEXEC, LIBGDAPREFIX)) 
-			prefix = g_strdup (LIBGDALIBEXEC);
+		tmp = LIBGDALIBEXEC;
+#ifndef G_OS_WIN32
+		if (! g_str_has_prefix (tmp, LIBGDAPREFIX) || (tmp [prefix_len] != G_DIR_SEPARATOR)) 
+			prefix = g_strdup (tmp);
 		else
-			prefix_dir_name = "libexec";
+			prefix_dir_name = tmp + prefix_len + 1;
+#else
+		prefix_dir_name = "libexec";
+#endif
 		break;
 	case GDA_ETC_DIR:
-		if (! g_str_has_prefix (LIBGDASYSCONF, LIBGDAPREFIX)) 
-			prefix = g_strdup (LIBGDASYSCONF);
+		tmp = LIBGDASYSCONF;
+#ifndef G_OS_WIN32
+		if (! g_str_has_prefix (tmp, LIBGDAPREFIX) || (tmp [prefix_len] != G_DIR_SEPARATOR)) 
+			prefix = g_strdup (tmp);
 		else
-			prefix_dir_name = "etc";
+			prefix_dir_name = tmp + prefix_len + 1;
+#else
+		prefix_dir_name = "etc";
+#endif
 		break;
 	}
 
@@ -130,7 +179,6 @@ gda_gbr_get_file_path (GdaPrefixDir where, ...)
 	g_print ("%s ()\n", __FUNCTION__);
 #endif
 
-
 	if (!prefix) {
 		/* prefix part for each OS */
 #ifdef G_OS_WIN32
@@ -143,9 +191,10 @@ gda_gbr_get_file_path (GdaPrefixDir where, ...)
 		p = strrchr (prefix, G_DIR_SEPARATOR);
 		if (p && (g_ascii_strcasecmp (p + 1, "bin") == 0 ||
 			  g_ascii_strcasecmp (p + 1, "lib") == 0))
-			*p = '\0';	
+			*p = '\0';
 #elif HAVE_CARBON
-#define MAXLEN 500
+  #ifdef ENABLE_BINRELOC
+    #define MAXLEN 500
 		ProcessSerialNumber myProcess;
 		FSRef bundleLocation;
 		unsigned char bundlePath[MAXLEN];
@@ -153,15 +202,30 @@ gda_gbr_get_file_path (GdaPrefixDir where, ...)
 		if ((GetCurrentProcess (&myProcess) == noErr) &&
 		    (GetProcessBundleLocation (&myProcess, &bundleLocation) == noErr) &&
 		    (FSRefMakePath (&bundleLocation, bundlePath, MAXLEN) == noErr)) {
-			prefix = g_path_get_dirname ((const char*) bundlePath);
-			if (g_str_has_suffix (prefix, "bin"))
-				prefix [strlen (prefix) - 3] = 0;
+			if (g_str_has_suffix ((const gchar*) bundlePath, ".app"))
+				prefix = g_strdup_printf ("%s/Contents/Resources", (const gchar*) bundlePath);
+			else {
+				prefix = g_path_get_dirname ((const char*) bundlePath);
+				if (g_str_has_suffix (prefix, "bin"))
+					prefix [strlen (prefix) - 3] = 0;
+			}
+#ifdef GDA_DEBUG_NO
+			g_print ("BUNDLE=%s, prefix=%s\n", bundlePath, prefix);
+#endif
 		}
 		else
 			g_warning ("Could not get PREFIX (using Mac OS X Carbon)");
+  #else
+#ifdef GDA_DEBUG_NO
+		g_print ("Binreloc disabled, using %s\n", LIBGDAPREFIX);
+#endif
+  #endif
+
+		if (!prefix)
+			prefix = g_strdup (LIBGDAPREFIX);
 #else
 		if (!prefix)
-			prefix = _gda_gbr_find_prefix (LIBGDAPREFIX);
+		 	prefix = _gda_gbr_find_prefix (LIBGDAPREFIX);
 #endif
 	}
 
@@ -195,14 +259,24 @@ gda_gbr_get_file_path (GdaPrefixDir where, ...)
 	else
 		tmp = g_build_filename (prefix, file_part, NULL);
 
-	if (!g_file_test (tmp, G_FILE_TEST_EXISTS) &&
-	    g_str_has_suffix (prefix, "libgda")) {
-		/* test if we are in the sources */
-		g_free (tmp);
-		if (prefix_dir_name)
-			tmp = g_build_filename (LIBGDAPREFIX, prefix_dir_name, file_part, NULL);
-		else
-			tmp = g_build_filename (LIBGDAPREFIX, file_part, NULL);
+	if (!g_file_test (tmp, G_FILE_TEST_EXISTS)) {
+		if (g_str_has_suffix (prefix, "libgda")) {
+			/* test if we are in the sources */
+			g_free (tmp);
+			if (prefix_dir_name)
+				tmp = g_build_filename (LIBGDAPREFIX, prefix_dir_name, file_part, NULL);
+			else
+				tmp = g_build_filename (LIBGDAPREFIX, file_part, NULL);
+		}
+		else {
+			g_free (prefix);
+			prefix = g_strdup (LIBGDAPREFIX);
+			g_free (tmp);
+			if (prefix_dir_name)
+				tmp = g_build_filename (prefix, prefix_dir_name, file_part, NULL);
+			else
+				tmp = g_build_filename (prefix, file_part, NULL);
+		}
 	}
 
 	g_free (prefix);
